local ResourceLoader = {
    images = {},
    music = {},
    sounds = {},
    fonts = {},
    toLoad = {}
};

local finished = false;

---
-- Register a target to load resources.
--
-- @param target - The target to register.
--
function ResourceLoader:register(target)
    self.toLoad[#self.toLoad + 1] = target;
end

---
-- Loads all resources. This includes images, sounds, music
-- and fonts.
--
function ResourceLoader:loadResources()
    local startTime = love.timer.getTime();
    print("\n---- LOADING RESOURCES ----");
    print("Loading images: ");
    for i = 1, #self.toLoad do
        local target = self.toLoad[i];

        if target.loadImages then
            target.loadImages(self);
        end
    end

    print("Loading sounds: ");
    for i = 1, #self.toLoad do
        local target = self.toLoad[i];
        if target.loadSounds then
            target.loadSounds(self);
        end
    end

    print("Loading music: ");
    for i = 1, #self.toLoad do
        local target = self.toLoad[i];

        if target.loadMusic then
            target.loadMusic(self);
        end
    end

    print("Loading fonts: ");
    for i = 1, #self.toLoad do
        local target = self.toLoad[i];

        if target.loadFonts then
            target.loadFonts(self);
        end
    end

    local endTime = love.timer.getTime();
    print(string.format("\nResources loaded in %.3f ms!", 1000 * (endTime - startTime)))
    print("===================");

    finished = true;
end

---
-- Loads an image and stores it in the resource manager's
-- image table. If the same image is already stored in that
-- table that one will be returned. This way no resource will be
-- loaded twice.
--
-- @param path - The path to the image to load.
--
function ResourceLoader:loadImage(path)
    if not self.images[path] then
        self.images[path] = love.graphics.newImage(path);
        print("    " .. path);
    end
    return self.images[path];
end

---
-- Loads a sound and stores it in the sounds table.
--
-- @param path - The path to the sound to load.
-- @param type - The sound type ('stream' or 'static').
--
function ResourceLoader:loadSound(path, type)
    if not self.sounds[path] then
        self.sounds[path] = love.audio.newSource(path, type)
        print("    " .. path)
    end

    return self.sounds[path]
end

---
-- Loads a song and stores it in the music table.
--
-- @param path - The path to the song to load.
-- @param type - The sound type ('stream' or 'static').
--
function ResourceLoader:loadMusic(path, type)
    if not self.music[path] then
        self.music[path] = love.audio.newSource(path, type)
        print("    " .. path)
    end

    return self.music[path]
end

---
-- Loads a font and stores it in the fonts table.
--
-- @param path - The path to the font to load.
-- @param size - The size to set the font.
--
function ResourceLoader:loadFont(path, size)
    if not self.fonts[path] then
        self.fonts[path] = love.graphics.newFont(path, size)
        print("    " .. path)
    end

    return self.fonts[path]
end

function ResourceLoader:isDone()
    return finished;
end

return ResourceLoader;